@@ -1,4 +1,3 @@
-#include <locale.h>
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 #include <libgnome-desktop/gnome-wall-clock.h>
 
@@ -18,10 +17,10 @@ int main (int argc, char **argv)
 	GMainLoop *loop;
 	GnomeWallClock *clock;
 
-        setlocale (LC_ALL, "");
-
 	g_setenv ("G_MESSAGES_DEBUG", "all", TRUE);
 
+	g_type_init ();
+
 	loop = g_main_loop_new (NULL, FALSE);
 
 	clock = g_object_new (GNOME_TYPE_WALL_CLOCK, NULL);
